from bn import AttributeDict
from httpkit.helper.http import HTTPResponse
from commandtool import to_internal, Cmd
import getopt

def wsgi_handler(service, start_services, host='', port=8000, profile=False):
    """Start a server with the ``app``, ``host`` and ``port`` specified."""
    def app(environ, start_response):
        new_service = AttributeDict()
        new_service['http'] = AttributeDict(response=HTTPResponse(),environ= environ)
        service.boot.start_flow(new_service, start_services)
        start_response(
            new_service.http.response.status, 
            new_service.http.response.headers
        )
        return new_service.http.response.get_result()
    if profile:
        # pstats problems:
        # sudo apt-get install python-profiler
        #  ~/env/bin/easy_install repoze.profile
        from repoze.profile.profiler import AccumulatingProfileMiddleware
        app = AccumulatingProfileMiddleware(
            app,
            log_filename='profile.log',
            cachegrind_filename='profile.cachegrind',
            discard_first_request=True,
            flush_at_shutdown=True,
            path='/__profile__' 
        )
    try:
        from paste.httpserver import serve
    except:
        from wsgiref.simple_server import make_server
        httpd = make_server(host, port, app)
        print "Serving HTTP with wsgiref on http://%s:%s/"%(host, port)
        httpd.serve_forever()
    else:
        print "Serving HTTP with Paste"
        serve(app, host, port)


class ServeCmd(Cmd):

    arg_spec = [
       #('architecture', "can be `wsgi' or `tornado'"),
    ]
    # The option_spec specifies which options are associated with 
    # each internal variable
    option_spec = {
        'help': dict(
            options = ['-h', '--help'],
            help = 'display this message'
        ),
    }
    help = {
        # If you leave start and end as None, values will be 
        # automatically generated
        'start': None,
        'end': None,
        # Used in the main command help to describe this command
    }

    def __init__(
        self, 
        start_services, 
        handler=wsgi_handler, 
        help_summary='Serve the application'
    ):
        self.start_services = start_services
        self.handler = handler
        self.help['summary'] = help_summary

    def on_run(self, service, args, opts):

        from conversionkit import chainConverters, noConversion, Conversion
        from stringconvert import unicodeToInteger, unicodeToBoolean
        from recordconvert import toRecord

        if not service.boot.option.has_key('server'):
            handle_section_error(None, 'server', "'server.host'")
        server_converters={
            'host': noConversion(),
            'port': unicodeToInteger(),
            'profile': unicodeToBoolean(),
            'display_host': noConversion(),
            'display_port': unicodeToInteger(),
        }
        server_converter = chainConverters(
            toRecord(
                missing_errors=[
                    "The required option 'server.%(key)s' is missing", 
                    ('host', 'port'),
                ],
                missing_defaults={
                    "profile": False,
                },
                empty_errors="The option 'server.%(key)s' cannot be empty",
                converters=server_converters,
            ),
        )
        conversion = Conversion(service.boot.option['server']).perform(
            server_converter
        )
        if not conversion.successful:
            handle_option_error(conversion, 'server')
        else:
            service.config['server'] = conversion.result
        self.handler(
            service,
            self.start_services,
            host = service.config.server.host,
            port = service.config.server.port,
            profile = service.config.server.profile,
        )
        return 0

def serveWsgi(start_services):
    return ServeCmd(
        start_services, 
        wsgi_handler, 
        'Serve the application using a threaded WSGI server (paste.httpserver if possible, otherwise wsgiref)'
    )

